/**
 * Implementierung eines binren Suchbaums
 *
 * Beachte: 
 * Der Suchbaum ist kein ausgeglichener Baum.
 * Er kann durch Einfge- und Lschoperationen zu einer
 * linearen Liste entarten.
 *
 * Die gespeicherten Objektreferenzen fr Werte drfen 
 * nicht gleich null sein.
 *
 */
public class MyTreeMap
{
	/** 
	 * Innere Klasse fr die Knoten des binren Sucbaumes
	 */
	class Node 
	{
		// Attribute 
		Comparable key; // Verweis auf Schlssel-Objekt
		Object value;   // Verweis auf Wert-Objekt
		Node left;      // Verweis auf linken Kindknoten
		Node right;     // Verweis auf rechten Kindknoten
		
		// Konstruktor
		Node(Comparable key, Object value)
		{
	    	this.key = key;
	    	this.value = value;
	    	this.left = null;
	    	this.right = null;
	    }
	    
	    public String toString()
	    {
	    	return "(" + key + ", " + value + ")";
	    }
    }

	// Attribute (Klasse MyTreeMap)
	private Node root;   // Verweis auf Wurzelknoten des Binrbaumes
	private int size;    // Anzahl der Knoten des Binrbaumes
	
	// Konstruktor (Klasse MyTreeMap)
	/**
	 * Erzeugung eines leeren Binrbaums
	 */
	public MyTreeMap()
	{
		this.root = null;
		this.size = 0;
	}

	// Operationen (Klasse MyTreeMap)
	/**
	 * Bestimmung der Anzahl der Knoten im Binrbaum
	 */
	public int size()
	{
		return this.size;
	}

	/*
	 * Bestimmung eines Knotens, der einen vorgegebenen Schlssel enthlt
	 * mit Hilfe eines rekursiven Algorithmus
	 * Die Operation setzt voraus, dass key ungleich null ist. 
	 */	
	private Node getNodeRec(Comparable key, Node tree) 
	{
		Node result;
		
		if (tree==null)			// Baum ist leer
			result = null;
		else
		{
			int cmp = key.compareTo(tree.key);
			if (cmp==0)			// Knoten ist gefunden
			    result = tree;
			else if(cmp<0)      // Suchen im linken Teilbaum
				result = getNodeRec(key, tree.left);
			else			    // Suchen im rechten Teilbaum
				result = getNodeRec(key, tree.right);
		}
		
		return result;
	}

	/*
	 * Bestimmung eines Knotens, der einen vorgegebenen Schlssel enthlt
	 * mit Hilfe eines iterativen Algorithmus
	 * Die Operation setzt voraus, dass key ungleich null ist. 
	 */	
	private Node getNodeIt(Comparable key, Node tree) 
	{
		Node result;
		
		result = null;


		// Hier Lsung einfgen

		
		return result;
	}
 
	/**
	 * Bestimmung des Wertes zu einem vorgegebenen Schlssel 
	 */	
	Object get(Comparable key) 
	{
		Object result = null;
		
		if (key!=null)
		{
			Node p = getNodeRec(key, this.root);
			if (p!=null)
				result = p.value;
		}

   		return result;
	}

	/**
	 * berprfung, ob ein zu o inhaltsgleiches Objekt als Werte-Objekt
	 * enthalten ist
	 */	
	boolean containsValue(Object o) 
	{
		boolean result = false;

		// Hier Lsung einfgen
		
   		return result;
	}

	
	/*
	 * Rekursives Verfahren zum Einfgen eines Paares (key,value) 
	 * in den Binrbaum mit dem Wurzelknoten tree.
	 * Ist der Schlssel key schon vorhanden, wird der alte
	 * Wert durch den Wert value ersetzt.
	 * Als Ergebnis wir ein Baum zurckgeliefert, der das 
	 * Paar (key, value) enthlt.
	 */	
	private Node insertNode(Comparable key, Object value, Node tree) 
	{
		if (tree==null)
		{						// Neuen Knoten erzeugen
			tree = new Node(key, value);
			this.size++;
		}
		else
		{
			int cmp = key.compareTo(tree.key);
			if (cmp==0)
			{
				// Alter Wert wir durch neuen Wert ersetzt
				tree.value = value;
			}
			else if(cmp<0)      // Einfgen im linken Teilbaum
				tree.left = insertNode(key, value, tree.left);
			else			    // Einfgen im rechten Teilbaum
				tree.right = insertNode(key, value, tree.right);
		}
		
		return tree;
	}

	/**
	 * Einfgen eines Paares (key,value) in den Binrbaum
	 * Ist der Schlssel key schon vorhanden, wird der alte
	 * Wert durch den Wert value ersetzt.
	 */
	void put(Comparable key, Object value)
	{
		if (key!=null && value!=null)
			this.root = insertNode(key, value, this.root);
	}	
	
	/*
	 * Bestimmung des Elternknotens des symmetrischen Nachfolgers
	 * des Knotens tree
	 *
	 * Der symmetrische Nachfolger eines Knotens, ist der Knoten
	 * mit dem kleinsten Schlssel, der grer als der Schlssel
	 * des Knotens tree ist.
	 * Dieser ist der am weitesten links stehende Knoten im rechten
	 * Teilbaum des Knotens tree.
	 */
	private Node parentSymSucc(Node tree)
	{
		Node result;
		
		result = tree;
		if (result.right.left!=null)
		{
			result = result.right;
			while (result.left.left!=null)
				result = result.left;
		}
		
		return result;
	}

	/*
	 * Rekursives Verfahren zum Entfernen eines Knotens zu einem
	 * vorgegebenen Schlssel im Binrbaum mit dem Wurzelknoten tree.
	 * Als Ergebnis wir ein Baum zurckgeliefert, der den Schlssel 
	 * key nicht mehr enthlt.
	 */				
	private Node removeNode(Comparable key, Node tree)
	{
		if (tree!=null)
		{
			int cmp = key.compareTo(tree.key);
			if (cmp<0)		// Entfernen im linken Teilbaum
				tree.left = removeNode(key, tree.left);
			else if (cmp>0)	// Entfernen im rechten Teilbaum
				tree.right = removeNode(key, tree.right);
			else
			{
				// zu entfernender Knoten gefunden
				this.size--;
				if (tree.left==null)
					tree = tree.right;	// Fall 1: siehe Skript
				else if (tree.right==null)
					tree = tree.left;	// Fall 2: siehe Skript
				else
				{
					// Knoten besitzt zwei Kindknoten
					Node p = parentSymSucc(tree);
					if (p==tree)		// Fall 3: siehe Skript
					{
						tree.key = p.right.key;
						tree.value = p.right.value;
						p.right = p.right.right;
					}					// Fall 4: siehe Skript
					else
					{
						tree.key = p.left.key;
						tree.value = p.left.value;
						p.left = p.left.right;
					}
				}
			}			
		}
		
		return tree;
	}
	
	/**
	 * Entfernen eines Eintrags im Binrbaum zu einem vorgegebenen Schlssel
	 */
	void remove(Comparable key)
	{
		if (key!=null)
			this.root = removeNode(key, this.root);
	}
	
	/*
	 * Rekursives Verfahren zur Ausgabe der Paare (Schlssel,Wert)
	 * sortiert nach aufsteigender Reihenfolge der Schlsseln
	 * durch einen Inorder-Durchlauf durch den Binrbaum
	 */
	private void inorderPrint(Node tree)
	{
		if (tree!=null)
		{
			inorderPrint(tree.left);
			System.out.println(tree);
			inorderPrint(tree.right);
	    }
	} 
	
	/** 
     * Ausgabe der Paare (Schlssel,Wert) sortiert nach aufsteigender
     * Reihenfolge der Schlssel
     */ 	 
	void print()
	{
		System.out.println("Liste der Eintrge");
		System.out.println("------------------");
		inorderPrint(this.root);
		System.out.println();
	}
}
